import {
  select,
  timer,
  geoOrthographic,
  geoPath,
  geoGraticule,
  geoDistance,
  json,
  tsv,
  zoom,
  event
} from 'd3';

// Rotation Settings
const config = {
  speed: 0.001,
  verticalTilt: -30,
  horizontalTilt: 0
}

// Select the SVG, append a Group and zoom
const svg = select('svg');
const globeGroup = svg.append('g');
const staticGroup = globeGroup.append('g');
const rotateGroup = globeGroup.append('g');
svg.call(zoom().on('zoom', () => {
  globeGroup.attr('transform', event.transform);
}));

// Setup the Geo Projection
const projection = geoOrthographic();
const path = geoPath().projection(projection);

drawGlobe();
enableRotation();    

function drawGlobe() {  
  Promise.all([
    json('https://unpkg.com/world-atlas@1.1.4/world/50m.json'),
    tsv('https://unpkg.com/world-atlas@1.1.4/world/50m.tsv')
  ]).then(([worldData,countryData]) => {
    
    // Build Lookup table for Country Names 
    const countryNames = countryData.reduce((a, d) => {
      a[d.iso_n3] = d.name;
      return a;
    }, {})

  // Draw the Ocean
  staticGroup.append('path')
    .attr('d', path({type: 'Sphere'}))
    .attr('class', 'ocean');

    // Draw the Meridians
    rotateGroup
      .append('path')
        .datum(geoGraticule()
          .step([10, 10]))
        .attr('class', 'graticule')
        .attr('d', path);

    // Draw Land Mass
    rotateGroup.selectAll('.land')
        .data(topojson.feature(worldData, worldData.objects.countries).features)
        .enter().append('path')
          .attr('class', 'land')
          .attr('d', path)
        .append('title')
          .text(d => countryNames[d.id]);
  });
}

function enableRotation() {
  timer(function (elapsed) {
    // Rotate the projection and apply the path to the SVG (Rotate Group)
    projection.rotate([config.speed * elapsed - 120, config.verticalTilt, config.horizontalTilt]);
    rotateGroup.selectAll('path').attr('d', path);
  });
}
